@@ -597,34 +597,24 @@ completion(const char *dbpath, const char *root, const char *prefix)
  */
 /*
  * print number of object.
+ *
+ * This procedure is commonly used except for the -P option.
  */
 void
 print_count(int number)
 {
-	if (format == FORMAT_PATH) {
-		switch (number) {
-		case 0:
-			fprintf(stderr, "object not found");
-			break;
-		case 1:
-			fprintf(stderr, "1 file located");
-			break;
-		default:
-			fprintf(stderr, "%d files located", number);
-			break;
-		}
-	} else {
-		switch (number) {
-		case 0:
-			fprintf(stderr, "object not found");
-			break;
-		case 1:
-			fprintf(stderr, "1 object located");
-			break;
-		default:
-			fprintf(stderr, "%d objects located", number);
-			break;
-		}
+	const char *target = format == FORMAT_PATH ? "file" : "object";
+
+	switch (number) {
+	case 0:
+		fprintf(stderr, "object not found");
+		break;
+	case 1:
+		fprintf(stderr, "1 %s located", target);
+		break;
+	default:
+		fprintf(stderr, "%d %ss located", number, target);
+		break;
 	}
 }
 /*
@@ -855,7 +845,7 @@ pathlist(const char *pattern, const char *dbpath)
 			fprintf(stderr, "file not found");
 			break;
 		case 1:
-			fprintf(stderr, "%d file located", count);
+			fprintf(stderr, "1 file located");
 			break;
 		default:
 			fprintf(stderr, "%d files located", count);
@@ -1057,6 +1047,7 @@ search(const char *pattern, const char *root, const char *cwd, const char *dbpat
 			count++;
 		} else if (gtop->format & GTAGS_COMPACT) {
 			/*
+			 * Compact format:
 			 *                    a          b
 			 * tagline = <file id> <tag name> <line no>,...
 			 */
@@ -1099,14 +1090,16 @@ search(const char *pattern, const char *root, const char *cwd, const char *dbpat
 			}
 			/*
 			 * Unfold compact format.
-			 *
-			 * If GTAGS_COMPLINE flag is set, each line number is expressed as
-			 * the difference from the previous line number except for the head.
-			 * Please see flush_pool() in libutil/gtagsop.c for the details.
 			 */
 			if (!isdigit(*p))
 				die("illegal compact format.");
 			if (gtop->format & GTAGS_COMPLINE) {
+				/*
+				 *
+				 * If GTAGS_COMPLINE flag is set, each line number is expressed as
+				 * the difference from the previous line number except for the head.
+				 * Please see flush_pool() in libutil/gtagsop.c for the details.
+				 */
 				int last = 0, cont = 0;
 
 				while (*p || cont > 0) {
@@ -1128,8 +1121,12 @@ search(const char *pattern, const char *root, const char *cwd, const char *dbpat
 					}
 					if (last_lineno != n && fp) {
 						while (lineno < n) {
-							if (!(src = strbuf_fgets(ib, fp, STRBUF_NOCRLF)))
-								die("unexpected end of file. '%s: %d/%d'", gtp->path, lineno, n);
+							if (!(src = strbuf_fgets(ib, fp, STRBUF_NOCRLF))) {
+								src = "";
+								fclose(fp);
+								fp = NULL;
+								break;
+							}
 							lineno++;
 						}
 					}
@@ -1140,6 +1137,11 @@ search(const char *pattern, const char *root, const char *cwd, const char *dbpat
 					last_lineno = last = n;
 				}
 			} else {
+				/*
+				 * In fact, when GTAGS_COMPACT is set, GTAGS_COMPLINE is allways set.
+				 * Therefore, the following code are not actually used.
+				 * However, it is left for some test.
+				 */
 				while (*p) {
 					for (n = 0; isdigit(*p); p++)
 						n = n * 10 + *p - '0';
@@ -1149,8 +1151,12 @@ search(const char *pattern, const char *root, const char *cwd, const char *dbpat
 						continue;
 					if (last_lineno != n && fp) {
 						while (lineno < n) {
-							if (!(src = strbuf_fgets(ib, fp, STRBUF_NOCRLF)))
-								die("unexpected end of file. '%s: %d/%d'", gtp->path, lineno, n);
+							if (!(src = strbuf_fgets(ib, fp, STRBUF_NOCRLF))) {
+								src = "";
+								fclose(fp);
+								fp = NULL;
+								break;
+							}
 							lineno++;
 						}
 					}
@@ -1163,6 +1169,7 @@ search(const char *pattern, const char *root, const char *cwd, const char *dbpat
 			}
 		} else {
 			/*
+			 * Standard format:
 			 *                    a          b         c
 			 * tagline = <file id> <tag name> <line no> <line image>
 			 */
