@@ -525,12 +525,10 @@ find_read_traverse(void)
 				 * o dead symbolic link
 				 */
 				if (!test("f", path)) {
-					if (!qflag) {
-						if (test("d", path))
-							warning("'%s' is a directory. (Ignored)", path);
-						else
-							warning("'%s' not found. (Ignored)", path);
-					}
+					if (test("d", path))
+						warning("'%s' is a directory. (Ignored)", path);
+					else
+						warning("'%s' not found. (Ignored)", path);
 					continue;
 				}
 				/*
@@ -538,8 +536,7 @@ find_read_traverse(void)
 				 * It will be improved in the future.
 				 */
 				if (locatestring(path, " ", MATCH_FIRST)) {
-					if (!qflag)
-						warning("'%s' ignored, because it includes blank.", &path[2]);
+					warning("'%s' ignored, because it includes blank.", &path[2]);
 					continue;
 				}
 				/*
@@ -624,12 +621,10 @@ find_read_filelist(void)
 		 * o dead symbolic link
 		 */
 		if (!test("f", path)) {
-			if (!qflag) {
-				if (test("d", path))
-					warning("'%s' is a directory. (Ignored)", path);
-				else
-					warning("'%s' not found. (Ignored)", path);
-			}
+			if (test("d", path))
+				warning("'%s' is a directory. (Ignored)", path);
+			else
+				warning("'%s' not found. (Ignored)", path);
 			continue;
 		}
 		/*
@@ -638,14 +633,17 @@ find_read_filelist(void)
 		 *	rootdir  /a/b/
 		 *	buf      /a/b/c/d.c -> c/d.c -> ./c/d.c
 		 */
-		path = normalize(path, rootdir, cwddir, buf, sizeof(buf));
+		if (normalize(path, rootdir, cwddir, buf, sizeof(buf)) == NULL) {
+			warning("'%s' is out of source tree. (Ignored)", path);
+			continue;
+		}
+		path = buf;
 		/*
 		 * GLOBAL cannot treat path which includes blanks.
 		 * It will be improved in the future.
 		 */
 		if (locatestring(path, " ", MATCH_LAST)) {
-			if (!qflag)
-				warning("'%s' ignored, because it includes blank.", path + 2);
+			warning("'%s' ignored, because it includes blank.", path + 2);
 			continue;
 		}
 		if (skipthisfile(path))
